﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Threading;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Networks.IntegratedOptimization.MachineLearning {
  [StorableClass]
  public sealed class OrchestratedBinaryProblem : SingleObjectiveBasicProblem<BinaryVectorEncoding> {

    [Storable]
    private readonly bool maximization;
    public override bool Maximization { get { return maximization; } }

    [Storable]
    public FeatureSelectionOrchestrator Orchestrator { get; private set; }

    [StorableConstructor]
    private OrchestratedBinaryProblem(bool deserializing) : base(deserializing) { }
    private OrchestratedBinaryProblem(OrchestratedBinaryProblem original, Cloner cloner)
      : base(original, cloner) {
      this.maximization = original.maximization;
      this.Orchestrator = cloner.Clone(original.Orchestrator);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new OrchestratedBinaryProblem(this, cloner);
    }

    public OrchestratedBinaryProblem(FeatureSelectionOrchestrator orchestator, int binaryVectorLength, bool maximization)
      : base() {
      Orchestrator = orchestator;
      Encoding.Length = binaryVectorLength;
      this.maximization = maximization;
    }

    public override double Evaluate(Individual individual, IRandom random) {
      var msg = Orchestrator.FeatureSelectionEvaluationPort.PrepareMessage();
      msg["BinaryVector"] = individual.BinaryVector();
      Orchestrator.FeatureSelectionEvaluationPort.ReceiveMessage(msg, new CancellationToken());
      var quality = (DoubleValue)msg["Quality"];
      return quality.Value;
    }
  }
}
